@@ -18,7 +18,7 @@ use indexmap::{IndexMap, IndexSet};
 const KNOWN_OPTIONS: [&str; 6] = ["-h", "-help", "--help", "-V", "-version", "--version"];
 
 pub(crate) struct Matcher<'a, 'b> {
-    cmds: Vec<(&'b str, &'a Command, String, usize)>,
+    cmds: Vec<LevelCommand<'a, 'b>>,
     args: &'b [String],
     flag_option_args: Vec<Vec<FlagOptionArg<'a, 'b>>>,
     positional_args: Vec<&'b str>,
@@ -28,12 +28,13 @@ pub(crate) struct Matcher<'a, 'b> {
     script_path: Option<String>,
     term_width: Option<usize>,
     split_last_arg_at: Option<usize>,
+    last_flag_option: Option<&'a str>,
 }
 
-type FlagOptionArg<'a, 'b> = (&'b str, Vec<&'b str>, Option<&'a str>);
-type LevelCommand<'a, 'b> = (&'b str, &'a Command, String, usize);
+type FlagOptionArg<'a, 'b> = (&'b str, Vec<&'b str>, Option<&'a str>); // key, values, param_name
+type LevelCommand<'a, 'b> = (&'b str, &'a Command, usize); // name, command, arg_index
 
-#[derive(Debug, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub(crate) enum ArgComp {
     FlagOrOption,
     FlagOrOptionCombine(String),
@@ -63,8 +64,9 @@ pub(crate) enum MatchError {
 }
 
 impl<'a, 'b> Matcher<'a, 'b> {
-    pub(crate) fn new(root: &'a Command, args: &'b [String]) -> Self {
-        let mut cmds: Vec<LevelCommand> = vec![(args[0].as_str(), root, args[0].clone(), 0)];
+    pub(crate) fn new(root: &'a Command, args: &'b [String], compgen: bool) -> Self {
+        let combine_shorts = root.metadata.contains_key("combine-shorts");
+        let mut cmds: Vec<LevelCommand> = vec![(args[0].as_str(), root, 0)];
         let mut cmd_level = 0;
         let mut arg_index = 1;
         let mut flag_option_args = vec![vec![]];
@@ -73,6 +75,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
         let mut split_last_arg_at = None;
         let mut arg_comp = ArgComp::Any;
         let mut choice_fns = HashSet::new();
+        let mut last_flag_option = None;
         let args_len = args.len();
         if root.delegated() {
             positional_args = args.iter().skip(1).map(|v| v.as_str()).collect();
@@ -88,6 +91,8 @@ impl<'a, 'b> Matcher<'a, 'b> {
             while arg_index < args_len {
                 let cmd = cmds[cmd_level].1;
                 let arg = args[arg_index].as_str();
+                let is_last_arg = arg_index == args_len - 1;
+                last_flag_option = None;
                 if arg == "--" {
                     if is_rest_args_positional {
                         add_positional_arg(
@@ -99,7 +104,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                     }
                     if dashes.is_none() {
                         dashes = Some(positional_args.len());
-                        if arg_index != args_len - 1 {
+                        if !is_last_arg {
                             is_rest_args_positional = true
                         }
                     }
@@ -116,11 +121,12 @@ impl<'a, 'b> Matcher<'a, 'b> {
                     if let Some((k, v)) = arg.split_once('=') {
                         if let Some(param) = cmd.find_flag_option(k) {
                             add_param_choice_fn(&mut choice_fns, param);
-                            if arg_index == args_len - 1 {
+                            if is_last_arg {
                                 arg_comp = ArgComp::OptionValue(param.name().to_string(), 0);
                                 split_last_arg_at = Some(k.len() + 1);
                             }
                             flag_option_args[cmd_level].push((k, vec![v], Some(param.name())));
+                            last_flag_option = Some(param.name());
                         } else if let Some((param, prefix)) = cmd.find_prefixed_option(arg) {
                             add_param_choice_fn(&mut choice_fns, param);
                             match_prefix_option(
@@ -132,6 +138,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                                 &mut split_last_arg_at,
                                 &prefix,
                             );
+                            last_flag_option = Some(param.name());
                         } else {
                             flag_option_args[cmd_level].push((k, vec![v], None));
                         }
@@ -144,7 +151,9 @@ impl<'a, 'b> Matcher<'a, 'b> {
                             param,
                             &mut arg_comp,
                             &mut split_last_arg_at,
+                            combine_shorts,
                         );
+                        last_flag_option = Some(param.name());
                     } else if let Some((param, prefix)) = cmd.find_prefixed_option(arg) {
                         add_param_choice_fn(&mut choice_fns, param);
                         match_prefix_option(
@@ -156,7 +165,16 @@ impl<'a, 'b> Matcher<'a, 'b> {
                             &mut split_last_arg_at,
                             &prefix,
                         );
-                    } else if let Some(subcmd) = find_subcommand(cmd, arg, &positional_args) {
+                        last_flag_option = Some(param.name());
+                    } else if let Some(subcmd) = find_subcommand(cmd, arg, &positional_args)
+                        .and_then(|v| {
+                            if is_last_arg && compgen {
+                                None
+                            } else {
+                                Some(v)
+                            }
+                        })
+                    {
                         match_command(
                             &mut cmds,
                             &mut cmd_level,
@@ -165,10 +183,9 @@ impl<'a, 'b> Matcher<'a, 'b> {
                             arg,
                             arg_index,
                         );
-                        if arg_index == args_len - 1 && arg.len() == 2 {
-                            arg_comp = ArgComp::FlagOrOptionCombine(arg.to_string());
-                        }
-                    } else if let Some((mut arr, maybe_subcmd)) = match_combine_shorts(cmd, arg) {
+                    } else if let Some((mut arr, maybe_subcmd)) =
+                        match_combine_shorts(cmd, arg, combine_shorts)
+                    {
                         let mut current_cmd = cmd;
                         if let Some(subcmd) = maybe_subcmd {
                             match_command(
@@ -181,22 +198,40 @@ impl<'a, 'b> Matcher<'a, 'b> {
                             );
                             current_cmd = subcmd;
                         }
-                        let name = arr.pop().and_then(|v| v.2).unwrap();
-                        let param = current_cmd.find_flag_option(name).unwrap();
-                        add_param_choice_fn(&mut choice_fns, param);
-                        flag_option_args[cmd_level].extend(arr);
-                        match_flag_option(
-                            &mut flag_option_args[cmd_level],
-                            args,
-                            &mut arg_index,
-                            param,
-                            &mut arg_comp,
-                            &mut split_last_arg_at,
-                        );
+
+                        if arr.is_empty() {
+                            // match subcommand
+                            if combine_shorts {
+                                arg_comp = ArgComp::FlagOrOptionCombine(arg.to_string());
+                            }
+                        } else {
+                            let name = arr.pop().and_then(|v| v.2).unwrap();
+                            let param = current_cmd.find_flag_option(name).unwrap();
+                            add_param_choice_fn(&mut choice_fns, param);
+                            flag_option_args[cmd_level].extend(arr);
+                            match_flag_option(
+                                &mut flag_option_args[cmd_level],
+                                args,
+                                &mut arg_index,
+                                param,
+                                &mut arg_comp,
+                                &mut split_last_arg_at,
+                                combine_shorts,
+                            );
+                            last_flag_option = Some(param.name());
+                        }
                     } else {
                         flag_option_args[cmd_level].push((arg, vec![], None));
                     }
-                } else if let Some(subcmd) = find_subcommand(cmd, arg, &positional_args) {
+                } else if let Some(subcmd) =
+                    find_subcommand(cmd, arg, &positional_args).and_then(|v| {
+                        if is_last_arg && compgen {
+                            None
+                        } else {
+                            Some(v)
+                        }
+                    })
+                {
                     match_command(
                         &mut cmds,
                         &mut cmd_level,
@@ -241,6 +276,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
             script_path: None,
             term_width: None,
             split_last_arg_at,
+            last_flag_option,
         }
     }
 
@@ -278,6 +314,12 @@ impl<'a, 'b> Matcher<'a, 'b> {
         if let Some(name) = &last_cmd.name {
             output.push(ArgcValue::Single("_cmd_fn".into(), name.to_string()));
         }
+        if let Some(name) = self.last_flag_option {
+            output.push(ArgcValue::Single(
+                "_last_flag_option".into(),
+                name.to_string(),
+            ));
+        }
         output
     }
 
@@ -291,11 +333,12 @@ impl<'a, 'b> Matcher<'a, 'b> {
             return vec![(
                 "__argc_value=path".into(),
                 String::new(),
+                false,
                 CompColor::of_value(),
             )];
         }
         let level = self.cmds.len() - 1;
-        let mut last_cmd = self.cmds[level].1;
+        let last_cmd = self.cmds[level].1;
         if last_cmd.delegated() {
             return last_cmd
                 .positional_params
@@ -310,19 +353,34 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 output
             }
             ArgComp::FlagOrOptionCombine(value) => {
-                let mut output: Vec<CompItem> = self
-                    .comp_flag_options()
-                    .iter()
-                    .filter_map(|(x, y, z)| {
-                        if x.len() == 2 && x != value {
-                            Some((format!("{value}{}", &x[1..]), y.to_string(), *z))
+                let mut output: Vec<CompItem> = vec![];
+                if value.len() == 2 && self.cmds[level].0 == value {
+                    output.extend(comp_subcomands(self.cmds[level - 1].1, true));
+                }
+                output.extend(self.comp_flag_options().iter().filter_map(
+                    |(v, description, nospace, comp_color)| {
+                        if v.len() == 2 && v != value {
+                            Some((
+                                format!("{value}{}", &v[1..]),
+                                description.to_string(),
+                                *nospace,
+                                *comp_color,
+                            ))
                         } else {
                             None
                         }
-                    })
-                    .collect();
+                    },
+                ));
                 if output.len() == 1 {
-                    output.insert(0, (value.to_string(), String::new(), CompColor::of_flag()));
+                    output.insert(
+                        0,
+                        (
+                            value.to_string(),
+                            String::new(),
+                            false,
+                            CompColor::of_flag(),
+                        ),
+                    );
                 }
                 output
             }
@@ -330,12 +388,6 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 if self.positional_args.len() == 2 && self.positional_args[0] == "help" {
                     return comp_subcomands(last_cmd, false);
                 }
-                if level > 0
-                    && self.positional_args.is_empty()
-                    && self.flag_option_args[level].is_empty()
-                {
-                    last_cmd = self.cmds[level - 1].1;
-                }
                 let values = self.match_positionals();
                 comp_subcommands_positional(last_cmd, &values, self.positional_args.len() < 2)
             }
@@ -365,6 +417,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
             output.push((
                 "__argc_value=path".into(),
                 String::new(),
+                false,
                 CompColor::of_value(),
             ));
         }
@@ -380,7 +433,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
         let cmds_len = self.cmds.len();
         let level = cmds_len - 1;
         let last_cmd = self.cmds[level].1;
-        let cmd_arg_index = self.cmds[level].3;
+        let cmd_arg_index = self.cmds[level].2;
         for level in 0..cmds_len {
             let args = &self.flag_option_args[level];
             let cmd = self.cmds[level].1;
@@ -548,19 +601,20 @@ impl<'a, 'b> Matcher<'a, 'b> {
                         return Some(MatchError::NotMultipleArgument(level, param.render_name()));
                     }
                     for values in values_list.iter() {
-                        if values.len() != param.values_size() {
-                            if param.is_flag() {
-                                return Some(MatchError::NoMoreValue(
-                                    level,
-                                    param.render_name(),
-                                    values[0].to_string(),
-                                ));
-                            } else if !param.multiple() {
-                                return Some(MatchError::MismatchValues(
-                                    level,
-                                    param.render_name_values(),
-                                ));
-                            }
+                        if param.is_flag() && !values.is_empty() {
+                            return Some(MatchError::NoMoreValue(
+                                level,
+                                param.render_name(),
+                                values[0].to_string(),
+                            ));
+                        } else if (param.ellipsis()
+                            && values.len() + 1 < param.arg_value_names.len())
+                            || (!param.ellipsis() && values.len() != param.arg_value_names.len())
+                        {
+                            return Some(MatchError::MismatchValues(
+                                level,
+                                param.render_name_values(),
+                            ));
                         }
                         if let Some(choices) =
                             get_param_choice(&param.data.choice, &choices_fn_values)
@@ -570,7 +624,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                                     return Some(MatchError::InvalidValue(
                                         level,
                                         value.to_string(),
-                                        param.render_single_value(),
+                                        param.render_first_value(),
                                         choices.clone(),
                                     ));
                                 }
@@ -778,12 +832,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
 
     fn get_cmd_and_paths(&self, level: usize) -> (&Command, Vec<&str>) {
         let cmd = self.cmds[level].1;
-        let cmd_paths: Vec<&str> = self
-            .cmds
-            .iter()
-            .take(level + 1)
-            .map(|v| v.2.as_str())
-            .collect();
+        let cmd_paths: Vec<&str> = self.cmds.iter().take(level + 1).map(|v| v.0).collect();
         (cmd, cmd_paths)
     }
 
@@ -801,7 +850,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
             if !last.is_empty() && param.is_match(last) {
                 exist = false;
             }
-            if !exist || param.multiple() {
+            if !exist || param.multi_occurs() {
                 let describe = param.describe_head();
                 let kind = if param.is_flag() {
                     CompColor::of_flag()
@@ -809,7 +858,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                     CompColor::of_option()
                 };
                 for v in param.list_names() {
-                    output.push((v, describe.to_string(), kind))
+                    output.push((v, describe.to_string(), param.prefixed().is_some(), kind))
                 }
             }
         }
@@ -817,7 +866,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
     }
 }
 
-pub(crate) type CompItem = (String, String, CompColor);
+pub(crate) type CompItem = (String, String, bool, CompColor);
 
 fn find_subcommand<'a>(
     cmd: &'a Command,
@@ -866,8 +915,12 @@ fn take_value_args(args: &[String], start: usize, len: usize) -> Vec<&str> {
 fn match_combine_shorts<'a, 'b>(
     cmd: &'a Command,
     arg: &'b str,
+    combine_shorts: bool,
 ) -> Option<(Vec<FlagOptionArg<'a, 'b>>, Option<&'a Command>)> {
-    if arg.starts_with("--") || arg.len() <= 2 {
+    if !combine_shorts {
+        return None;
+    }
+    if arg.starts_with("--") || arg == "-" {
         return None;
     }
     let mut current_cmd = cmd;
@@ -899,6 +952,7 @@ fn match_flag_option<'a, 'b>(
     param: &'a FlagOptionParam,
     arg_comp: &mut ArgComp,
     split_last_arg_at: &mut Option<usize>,
+    combine_shorts: bool,
 ) {
     if param.terminated() {
         let value_args: Vec<&str> = args[*arg_index + 1..].iter().map(|v| v.as_str()).collect();
@@ -910,7 +964,10 @@ fn match_flag_option<'a, 'b>(
         }
         flag_option_args.push((arg, value_args, Some(param.name())));
     } else {
-        let values_len = param.values_size();
+        let mut values_len = param.arg_value_names.len();
+        if param.ellipsis() {
+            values_len = usize::MAX / 2;
+        }
         let args_len = args.len();
         let value_args = take_value_args(args, *arg_index + 1, values_len);
         let arg = &args[*arg_index];
@@ -928,7 +985,7 @@ fn match_flag_option<'a, 'b>(
                     *arg_comp = ArgComp::OptionValue(param.name().to_string(), 0);
                     *split_last_arg_at = Some(prefix.len());
                 }
-            } else if param.is_flag() && !(arg.len() > 2 && param.is_match(arg)) {
+            } else if combine_shorts && param.is_flag() && !(arg.len() > 2 && param.is_match(arg)) {
                 *arg_comp = ArgComp::FlagOrOptionCombine(arg.to_string());
             }
         }
@@ -964,7 +1021,7 @@ fn match_command<'a, 'b>(
     arg_index: usize,
 ) {
     *cmd_level += 1;
-    cmds.push((arg, subcmd, arg.to_string(), arg_index));
+    cmds.push((arg, subcmd, arg_index));
     flag_option_args.push(vec![]);
 }
 
@@ -999,6 +1056,9 @@ fn comp_subcomands(cmd: &Command, flag: bool) -> Vec<CompItem> {
     for subcmd in cmd.subcommands.iter() {
         let describe = subcmd.describe_head();
         for v in subcmd.list_names() {
+            if v.len() < 2 {
+                continue;
+            }
             if (flag && v.starts_with('-')) || (!flag && !v.starts_with('-')) {
                 if !flag {
                     has_help_subcmd = true;
@@ -1006,7 +1066,7 @@ fn comp_subcomands(cmd: &Command, flag: bool) -> Vec<CompItem> {
                 if !describe.is_empty() {
                     describe_help_subcmd = true
                 }
-                output.push((v, describe.to_string(), CompColor::of_command()))
+                output.push((v, describe.to_string(), false, CompColor::of_command()))
             }
         }
     }
@@ -1019,6 +1079,7 @@ fn comp_subcomands(cmd: &Command, flag: bool) -> Vec<CompItem> {
         output.push((
             "help".to_string(),
             describe.to_string(),
+            false,
             CompColor::of_command(),
         ))
     }
@@ -1050,17 +1111,18 @@ fn comp_param(describe: &str, value_name: &str, data: &ParamData) -> Vec<CompIte
         match choices {
             Either::Left(choices) => choices
                 .iter()
-                .map(|v| (v.to_string(), String::new(), CompColor::of_value()))
+                .map(|v| (v.to_string(), String::new(), false, CompColor::of_value()))
                 .collect(),
             Either::Right(choices_fn) => vec![(
                 format!("__argc_fn={}", choices_fn),
                 String::new(),
+                false,
                 CompColor::of_value(),
             )],
         }
     } else {
         let value = format!("__argc_value={}", value_name);
-        vec![(value, describe.into(), CompColor::of_value())]
+        vec![(value, describe.into(), false, CompColor::of_value())]
     };
     if let Some(ch) = data.multi_char() {
         output.insert(
@@ -1068,6 +1130,7 @@ fn comp_param(describe: &str, value_name: &str, data: &ParamData) -> Vec<CompIte
             (
                 format!("__argc_multi={}", ch),
                 String::new(),
+                false,
                 CompColor::of_value(),
             ),
         );
