@@ -15,6 +15,7 @@ pub(crate) struct FlagOptionParam {
     pub(crate) data: ParamData,
     pub(crate) value_names: Vec<String>,
     pub(crate) arg_value_names: Vec<String>,
+    pub(crate) inherit: bool,
 }
 
 impl FlagOptionParam {
@@ -28,7 +29,9 @@ impl FlagOptionParam {
     ) -> Self {
         let name = param.name.clone();
         let value_names: Vec<String> = value_names.iter().map(|v| v.to_string()).collect();
-        let arg_value_names = if value_names.is_empty() {
+        let arg_value_names = if flag {
+            vec![]
+        } else if value_names.is_empty() {
             vec![to_cobol_case(&name)]
         } else {
             value_names.iter().map(|v| to_cobol_case(v)).collect()
@@ -41,6 +44,7 @@ impl FlagOptionParam {
             data: param,
             value_names,
             arg_value_names,
+            inherit: false,
         }
     }
 
@@ -120,7 +124,7 @@ impl FlagOptionParam {
         format!("{}{}", self.render_hyphens(), self.name())
     }
 
-    pub(crate) fn render_single_value(&self) -> String {
+    pub(crate) fn render_first_value(&self) -> String {
         format!("<{}>", self.arg_value_names[0])
     }
 
@@ -167,7 +171,7 @@ impl FlagOptionParam {
             let name: &String = &self.arg_value_names[0];
             let value = match (self.required(), self.multiple()) {
                 (true, true) => format!("<{name}>..."),
-                (false, true) => format!("[<{name}>...]"),
+                (false, true) => format!("[{name}]..."),
                 (_, false) => format!("<{name}>"),
             };
             output.push_str(&value);
@@ -177,7 +181,7 @@ impl FlagOptionParam {
                 .iter()
                 .enumerate()
                 .map(|(i, v)| {
-                    if self.multiple() && i == self.arg_value_names.len() - 1 {
+                    if self.ellipsis() && i == self.arg_value_names.len() - 1 {
                         format!("<{v}>...")
                     } else {
                         format!("<{v}>")
@@ -230,7 +234,7 @@ impl FlagOptionParam {
                         .collect()
                 }
                 Some(ArgcValue::Multiple(name, values))
-            } else if self.values_size() > 1 {
+            } else if self.arg_value_names.len() > 1 {
                 Some(ArgcValue::Multiple(
                     name,
                     values[0].iter().map(|v| v.to_string()).collect(),
@@ -246,7 +250,10 @@ impl FlagOptionParam {
     }
 
     pub(crate) fn prefixed(&self) -> Option<String> {
-        if self.data.modifer != Modifier::Prefixed {
+        if !matches!(
+            self.data.modifer,
+            Modifier::Prefixed | Modifier::MultiPrefixed
+        ) {
             return None;
         }
 
@@ -266,14 +273,12 @@ impl FlagOptionParam {
         output
     }
 
-    pub(crate) fn values_size(&self) -> usize {
-        if self.is_flag() {
-            0
-        } else if self.multiple() && self.arg_value_names.len() > 1 {
-            9999
-        } else {
-            self.arg_value_names.len()
-        }
+    pub(crate) fn multi_occurs(&self) -> bool {
+        self.multiple() && (self.flag || self.arg_value_names.len() == 1)
+    }
+
+    pub(crate) fn ellipsis(&self) -> bool {
+        self.terminated() || (self.multiple() && self.arg_value_names.len() > 1)
     }
 
     pub(crate) fn describe_head(&self) -> &str {
@@ -570,6 +575,7 @@ pub(crate) enum Modifier {
     MultiCharRequired(char),
     Terminated,
     Prefixed,
+    MultiPrefixed,
 }
 
 impl Modifier {
@@ -582,7 +588,8 @@ impl Modifier {
             Self::MultiCharOptional(_) => true,
             Self::MultiCharRequired(_) => true,
             Self::Terminated => true,
-            Self::Prefixed => true,
+            Self::Prefixed => false,
+            Self::MultiPrefixed => true,
         }
     }
 
@@ -596,6 +603,7 @@ impl Modifier {
             Self::MultiCharRequired(_) => true,
             Self::Terminated => false,
             Self::Prefixed => false,
+            Self::MultiPrefixed => false,
         }
     }
 
@@ -609,6 +617,7 @@ impl Modifier {
             Self::MultiCharRequired(c) => format!("+{c}"),
             Self::Terminated => "~".to_string(),
             Self::Prefixed => "-".to_string(),
+            Self::MultiPrefixed => "-*".to_string(),
         }
     }
 }
