@@ -32,6 +32,8 @@ pub(crate) enum EventData {
     Version(String),
     /// Author info
     Author(String),
+    /// Metadata
+    Meta(String, String),
     /// Define a subcommand, e.g. `@cmd A sub command`
     Cmd(String),
     /// Define alias for a subcommand, e.g. `@alias t,tst`
@@ -135,6 +137,7 @@ fn parse_tag(input: &str) -> nom::IResult<&str, Option<EventData>> {
         tuple((many1(char('#')), space0, char('@'))),
         alt((
             parse_tag_text,
+            parse_tag_meta,
             parse_tag_param,
             parse_tag_alias,
             parse_tag_unknown,
@@ -161,6 +164,15 @@ fn parse_tag_text(input: &str) -> nom::IResult<&str, Option<EventData>> {
     )(input)
 }
 
+fn parse_tag_meta(input: &str) -> nom::IResult<&str, Option<EventData>> {
+    preceded(
+        tag("meta"),
+        map(preceded(space1, parse_key_value), |kv| {
+            kv.map(|(k, v)| EventData::Meta(k.to_string(), v.to_string()))
+        }),
+    )(input)
+}
+
 fn parse_tag_param(input: &str) -> nom::IResult<&str, Option<EventData>> {
     let check = peek(alt((tag("option"), tag("flag"), tag("arg"))));
     let arg = alt((
@@ -348,11 +360,17 @@ fn parse_param_modifer(input: &str) -> nom::IResult<&str, ParamData> {
         map(
             pair(parse_param_name, preceded(tag("*"), opt(parse_multi_char))),
             |(mut arg, multi_char)| {
-                let modifier = match multi_char {
-                    Some(c) => Modifier::MultiCharOptional(c),
-                    None => Modifier::MultipleOptional,
+                match multi_char {
+                    Some(c) => arg.modifer = Modifier::MultiCharOptional(c),
+                    None => {
+                        if let Some(name) = arg.name.strip_suffix('-') {
+                            arg.name = name.to_string();
+                            arg.modifer = Modifier::MultiPrefixed
+                        } else {
+                            arg.modifer = Modifier::MultipleOptional
+                        }
+                    }
                 };
-                arg.modifer = modifier;
                 arg
             },
         ),
@@ -468,7 +486,10 @@ fn parse_zero_or_one_value_notation(input: &str) -> nom::IResult<&str, Option<&s
 
 // Parse '<FOO>'
 fn parse_value_notation(input: &str) -> nom::IResult<&str, &str> {
-    preceded(space0, delimited(char('<'), parse_notation_text, char('>')))(input)
+    preceded(
+        one_of(" "),
+        delimited(char('<'), parse_notation_text, char('>')),
+    )(input)
 }
 
 // Parse `a|b|c`
@@ -501,6 +522,19 @@ fn parse_tail(input: &str) -> nom::IResult<&str, &str> {
     ))(input)
 }
 
+fn parse_key_value(input: &str) -> nom::IResult<&str, Option<(&str, &str)>> {
+    let input = input.trim_end();
+    let key_value = alt((
+        map(
+            separated_pair(parse_name, char('='), terminated(parse_default_value, eof)),
+            |(key, value)| Some((key, value)),
+        ),
+        map(terminated(parse_name, eof), |key| Some((key, ""))),
+    ));
+
+    alt((key_value, success(None)))(input)
+}
+
 fn parse_name_list(input: &str) -> nom::IResult<&str, Vec<&str>> {
     separated_list1(char(','), delimited(space0, parse_name, space0))(input)
 }
@@ -687,6 +721,12 @@ mod tests {
                 Some(Some(EventData::$kind($text.to_string())))
             )
         };
+        ($comment:literal, Meta, $key:expr, $value:expr) => {
+            assert_eq!(
+                parse_line($comment).unwrap().1,
+                Some(Some(EventData::Meta($key.to_string(), $value.to_string())))
+            )
+        };
     }
 
     macro_rules! assert_parse_option_arg {
@@ -739,6 +779,8 @@ mod tests {
         assert_parse_option_arg!("--foo+,");
         assert_parse_option_arg!("--foo*,");
         assert_parse_option_arg!("--foo!");
+        assert_parse_option_arg!("--foo-*");
+        assert_parse_option_arg!("--foo-");
         assert_parse_option_arg!("--foo=a");
         assert_parse_option_arg!("--foo=`_foo`");
         assert_parse_option_arg!("--foo[a|b]");
@@ -773,6 +815,8 @@ mod tests {
         assert_parse_option_arg!("-foo+");
         assert_parse_option_arg!("-foo*");
         assert_parse_option_arg!("-foo!");
+        assert_parse_option_arg!("-foo-*");
+        assert_parse_option_arg!("-foo-");
         assert_parse_option_arg!("-foo=a");
         assert_parse_option_arg!("-foo=`_foo`");
         assert_parse_option_arg!("-foo[a|b]");
@@ -880,6 +924,8 @@ mod tests {
         assert_token!("# @describe A demo cli", Describe, "A demo cli");
         assert_token!("# @version 1.0.0", Version, "1.0.0");
         assert_token!("# @author Somebody", Author, "Somebody");
+        assert_token!("# @meta key", Meta, "key", "");
+        assert_token!("# @meta key=value", Meta, "key", "value");
         assert_token!("# @cmd A subcommand", Cmd, "A subcommand");
         assert_token!("# @alias tst", Aliases, ["tst"]);
         assert_token!("# @alias t,tst", Aliases, ["t", "tst"]);
