@@ -41,3 +41,13 @@ fn compgen() {
         .stdout(predicates::str::contains("abc\ndef\nghi"))
         .success();
 }
+
+#[test]
+fn compgen_argc() {
+    Command::cargo_bin("argc")
+        .unwrap()
+        .args(["--argc-compgen", "fish", "", "argc", "--argc-compgen", ""])
+        .assert()
+        .stdout(predicates::str::contains("zsh"))
+        .success();
+}
