@@ -4,7 +4,7 @@ use crate::*;
 fn multiple() {
     let script = r###"
 # @flag   -f --fc*
-# @option -o --oa* <DIR>
+# @option -a --oa* <DIR>
 # @option -b --ob* <CMD> <DIR>
 # @option -d --od <DIR> <FILE>
 # @arg var* <FILE>
@@ -19,9 +19,9 @@ fn multiple() {
             vec!["prog", "--", ""],
             vec!["prog", "-f", ""],
             vec!["prog", "--fc", ""],
-            vec!["prog", "-o", ""],
-            vec!["prog", "-o", "d1"],
-            vec!["prog", "-o", "d1", ""],
+            vec!["prog", "-a", ""],
+            vec!["prog", "-a", "d1"],
+            vec!["prog", "-a", "d1", ""],
             vec!["prog", "-b", ""],
             vec!["prog", "-b", "d1"],
             vec!["prog", "-b", "d1", ""],
@@ -42,6 +42,7 @@ fn multiple() {
 #[test]
 fn shorts() {
     const SCRIPT: &str = r###"
+# @meta combine-shorts
 # @flag   -a
 # @flag   -b --fb
 # @flag   -f --fc*
@@ -128,6 +129,7 @@ cmd::subb() { :; }
 #[test]
 fn flag_cmds() {
     const SCRIPT: &str = r###"
+# @meta combine-shorts
 # @option -G
 
 # @cmd Run --foo
@@ -154,7 +156,6 @@ bar() {
             vec!["prog", "-"],
             vec!["prog", "-B"],
             vec!["prog", "-B", "-"],
-            vec!["prog", "-BC"],
             vec!["prog", "-G"],
         ]
     );
@@ -346,6 +347,7 @@ cmdb() { :; }
 #[test]
 fn one_combine_shorts() {
     let script = r###"
+# @meta combine-shorts
 # @flag -a
 # @flag -b
 "###;
@@ -448,6 +450,32 @@ _choice_fn() {
     );
 }
 
+#[test]
+fn inherit_flag_options() {
+    let script = r###"
+# @meta inherit-flag-options
+# @flag --oa
+# @option --ob[a|b]  desc 1
+
+# @cmd
+cmda() {
+    :;
+}
+
+# @cmd
+# @option --ob[x|y]  desc 2
+cmdb() {
+    :;
+}
+"###;
+    snapshot_compgen!(
+        script,
+        [vec!["prog", "cmda", "--"], vec!["prog", "cmdb", "--"]]
+    );
+}
+
+// ------------ compgen shell -----------
+
 #[test]
 fn no_space() {
     let script = r#"
@@ -594,8 +622,9 @@ _choice_fn() {
 #[test]
 fn option_prefixed() {
     let script = r###"
-# @option -D-[`_choice_fn`]
-# @option -X --ox-[`_choice_fn`]
+# @option -D-*[`_choice_fn`]
+# @option -X --ox-*[`_choice_fn`]
+# @option -s-[`_choice_fn`]
 _choice_fn() {
     echo VAR1=value1 
     echo VAR2=value2
@@ -610,8 +639,11 @@ _choice_fn() {
             vec!["prog", "-DVAR1"],
             vec!["prog", "-X"],
             vec!["prog", "-XVAR1"],
+            vec!["prog", "-XVAR3", "-"],
             vec!["prog", "--ox"],
             vec!["prog", "--ox", "VAR1"],
+            vec!["prog", "-s"],
+            vec!["prog", "-sVAR3", "-"],
         ]
     );
 }
@@ -694,6 +726,17 @@ _choice_delegate() {
     );
 }
 
+#[test]
+fn cmd_aliases() {
+    let script = r###"
+# @cmd
+# @alias a, ab
+abc() { :; }
+"###;
+
+    snapshot_compgen!(script, [vec!["prog", ""]], argc::Shell::Bash);
+}
+
 #[test]
 fn mult_char() {
     let script = r#"
