@@ -189,7 +189,7 @@ fn option_to_variable() {
 #[test]
 fn option_terminated() {
     let script = r###"
-# @option --oa~ <SHELL> <SCRIPT> <ARGS...> 
+# @option --oa~ <SHELL> <SCRIPT> <ARGS>
 # @option --ob
 "###;
     snapshot!(script, &["prog", "--oa", "bash", "Argcfile.sh", "--ob"]);
@@ -198,9 +198,10 @@ fn option_terminated() {
 #[test]
 fn option_prefixed() {
     let script = r###"
-# @option -D-
+# @option -o-
+# @option -D-*
 "###;
-    snapshot!(script, &["prog", "-D", "v1", "-Dv2=foo"]);
+    snapshot!(script, &["prog", "-D", "v1", "-Dv2=foo", "-o1"]);
 }
 
 #[test]
@@ -228,6 +229,7 @@ bar() {
 #[test]
 fn cmd_combine_shorts() {
     let script = r###"
+# @meta combine-shorts
 # @cmd
 # @flag -B
 # @flag -C
@@ -247,3 +249,30 @@ fn name_with_special_chars() {
 "###;
     snapshot_multi!(script, [vec!["prog", "--fa:foo", "--fa.bar", "--fa_baz"]]);
 }
+
+#[test]
+fn inherit_flag_options() {
+    let script = r###"
+# @meta inherit-flag-options
+# @flag --oa
+# @option --ob[a|b]  desc 1
+
+# @cmd
+cmda() {
+    :;
+}
+
+# @cmd
+# @option --ob[x|y]  desc 2
+cmdb() {
+    :;
+}
+"###;
+    snapshot_multi!(
+        script,
+        [
+            vec!["prog", "cmda", "--ob", "a"],
+            vec!["prog", "cmdb", "--ob", "x"]
+        ]
+    );
+}
